# cleanup_aux_history.tree_filter.sh: 
#	A git tree-filter bash-script(cygwin) preparing Ricardo's repo to merge with VECTO-2.0 repo (central).
# 
# It is used by the cmd::
#		
#		git filter-branch ${TMP_DIR:+-d $TMP_DIR} --prune-empty --tree-filter  "$PWD/cleanup_aux_history-tree_filter.sh" Current
#
# Assumes a single repo having fetched branches from both repos:
#	- ricardo: cb11c45@Current
#
# by ankostis, 4-Dec-2015

set -o  errexit
shopt -s globstar


## Remove TeamFoundationServer bindings.
#
rm -f **/*.vssscc **/*.vspscc
for f in `find . -name '*.sln'`; do 
    res="`gawk '
        START                                           {del=0}
        /GlobalSection\(TeamFoundationVersionControl/   {del=1}
        del != 1                                        {print $0}
        /EndGlobalSection/                              {del=0}
    ' $f`"
    echo "$res" | unix2dos > $f
done

## Remove compiled artifacts.
#
find . -name bin -type d -print0|xargs -0 rm -rf


## /New Folder: A short-lived experiment.
# Completely discarded from history.
#
rm -rf "New Folder"


## /vecto-sim-ricardoaea: Original sources, never touched.
# Completely discarded from history, to use originals.
rm -rf vecto-sim-ricardoaea

## WHY??
# if [ -d "Third Party Libraries" ]; then
    # mkdir -p packages
    # mv "Third Party Libraries/*" packages/.
# fi

## /BuildProcessTemplates: Work-files needed once for commencing solution.
# To be archived in /Tools.
#
if [ -d BuildProcessTemplates ]; then
	mkdir -p Tools/Ricardo
	mv BuildProcessTemplates Tools/Ricardo/.
fi


## /PersistanceTB: Work-files needed once not sure why?
# To be archived in /Tools.
#
if [ -d PersistanceTB ]; then
	mkdir -p Tools/Ricardo
	mv PersistanceTB Tools/Ricardo/.
fi


## /VectoAuxiliaries: Development of `VECTOAux` started in this folder, 
#	and eventually Dana Nurse moved with 5d6fafd2e5532a@auxmerge/VectoAuxiliariesTB_begin
#   on Tue Sep 16 13:25:00 2014 +0000: Created Branch.
#   (But it continued to exist, as a relic).
#
#     $ git show-ref --tags |grep VectoAuxiliaries
#     5d6fafd2e5532ae6ae7554cb8d5375e313ac9be5 refs/tags/auxmerge/VectoAuxiliariesTB_begin
#     80fd13a4b739a4cd6f53cfc1689a4b6986c27ce7 refs/tags/auxmerge/VectoAuxiliaries_end
#     538e211c6566ce7c4a1737fecef24b3112f3cb0c refs/tags/auxmerge/VectoAuxiliaries_begin
#
#     $ git log  --oneline Current -- VectoAuxiliaries
#     9192214 Remove TFS bindings
#     96343b5 WIP
#     121916b git-tfs-id: [http://tfs00.element.root.com:8080/tfs/TFSCollection]$/VECTO;C1608
#     80fd13a Adding HVAC Tests
#     ca9c088 git-tfs-id: [http://tfs00.element.root.com:8080/tfs/TFSCollection]$/VECTO;C1602
#     93fe88c Started HVAC Load Demand Tests
#     7ef3936 Added HnacMap tests
#     538e211 First Check-In ready for working with new contractor
#
# So attaching `VectoAuxiliaries` as predecessor of `VECTOAux`, 
# and ignore all later changes.
# Specifically ignoring 96343b5 where Terry probably resurrected by mistake (temporarily) 
# development in this folder.
#
if [ -d VectoAuxiliaries -a  ! -d VectoAuxiliaries-TB ]; then
    mv VectoAuxiliaries VECTOAux
fi
rm -rf VectoAuxiliaries	

## /VectoAuxiliaries-TB: The main AdvancedAUX UI and code
# Rename it as `VECTOAux`.
#
if [ -d VectoAuxiliaries-TB ]; then
	mv VectoAuxiliaries-TB/ VECTOAux
fi
grep 'VectoAuxiliaries-TB' -rlZ * | xargs -r0I XXX sed 's/..\\VectoAuxiliaries-TB/VECTOAux/g' -i XXX

## /vecto-sim-ricardoaeaTB: VECTO-core modified by Ricardo
# Move its contents to the root of the project.
#
if [ -d vecto-sim-ricardoaeaTB ]; then
	rsync -ma vecto-sim-ricardoaeaTB/  .
    rm -rf vecto-sim-ricardoaeaTB
fi
grep 'vecto-sim-ricardoaeaTB' -rlZ * | xargs -r0I XXX sed 's/vecto-sim-ricardoaeaTB\\//g' -i XXX


## Ensure .gitignore
if [ ! -f .gitignore ]; then
    cat > .gitignore << EOF
VECTO/bin/
VECTO/obj/

## From github on 15/5/2014: https://github.com/github/gitignore/blob/master/VisualStudio.gitignore

## Ignore Visual Studio temporary files, build results, and
## files generated by popular Visual Studio add-ons.

# User-specific files
*.suo
*.user
*.sln.docstates

# Build results
[Dd]ebug/
[Dd]ebugPublic/
[Rr]elease/
[Rr]eleases/
x64/
build/
bld/
[Bb]in/
[Oo]bj/

# Roslyn cache directories
*.ide/

# MSTest test Results
[Tt]est[Rr]esult*/
[Bb]uild[Ll]og.*

#NUNIT
*.VisualState.xml
TestResult.xml

# Build Results of an ATL Project
[Dd]ebugPS/
[Rr]eleasePS/
dlldata.c

*_i.c
*_p.c
*_i.h
*.ilk
*.meta
*.obj
*.pch
*.pdb
*.pgc
*.pgd
*.rsp
*.sbr
*.tlb
*.tli
*.tlh
*.tmp
*.tmp_proj
*.log
*.vspscc
*.vssscc
.builds
*.pidb
*.svclog
*.scc

# Chutzpah Test files
_Chutzpah*

# Visual C++ cache files
ipch/
*.aps
*.ncb
*.opensdf
*.sdf
*.cachefile

# Visual Studio profiler
*.psess
*.vsp
*.vspx

# TFS 2012 Local Workspace
$tf/

# Guidance Automation Toolkit
*.gpState

# ReSharper is a .NET coding add-in
_ReSharper*/
*.[Rr]e[Ss]harper
*.DotSettings.user

# JustCode is a .NET coding addin-in
.JustCode

# TeamCity is a build add-in
_TeamCity*

# DotCover is a Code Coverage Tool
*.dotCover

# NCrunch
_NCrunch_*
.*crunch*.local.xml

# MightyMoose
*.mm.*
AutoTest.Net/

# Web workbench (sass)
.sass-cache/

# Installshield output folder
[Ee]xpress/

# DocProject is a documentation generator add-in
DocProject/buildhelp/
DocProject/Help/*.HxT
DocProject/Help/*.HxC
DocProject/Help/*.hhc
DocProject/Help/*.hhk
DocProject/Help/*.hhp
DocProject/Help/Html2
DocProject/Help/html

# Click-Once directory
publish/

# Publish Web Output
*.[Pp]ublish.xml
*.azurePubxml
## TODO: Comment the next line if you want to checkin your
## web deploy settings but do note that will include unencrypted
## passwords
*.pubxml

# NuGet Packages Directory
packages/*
## TODO: If the tool you use requires repositories.config
## uncomment the next line
#!packages/repositories.config

# Enable "build/" folder in the NuGet Packages folder since
# NuGet packages use it for MSBuild targets.
# This line needs to be after the ignore of the build folder
# (and the packages folder if the line above has been uncommented)
!packages/build/

# Windows Azure Build Output
csx/
*.build.csdef

# Windows Store app package directory
AppPackages/

# Others
sql/
*.Cache
ClientBin/
[Ss]tyle[Cc]op.*
~$*
*~
*.dbmdl
*.dbproj.schemaview
*.pfx
*.publishsettings
node_modules/

# RIA/Silverlight projects
Generated_Code/

# Backup & report files from converting an old project file
# to a newer Visual Studio version. Backup files are not needed,
# because we have git ;-)
_UpgradeReport_Files/
Backup*/
UpgradeLog*.XML
UpgradeLog*.htm

# SQL Server files
*.mdf
*.ldf

# Business Intelligence projects
*.rdl.data
*.bim.layout
*.bim_*.settings

# Microsoft Fakes
FakesAssemblies/
EOF
fi